package tofly.zggas.zggaspay.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.Generator;
import com.opencsv.CSVReader;
import com.wechat.pay.java.core.Config;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.exception.ServiceException;
import com.wechat.pay.java.core.notification.NotificationConfig;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RSANotificationConfig;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.certificate.CertificateService;
import com.wechat.pay.java.service.payments.model.DownloadDetail;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.payments.nativepay.NativePayService;
import com.wechat.pay.java.service.payments.nativepay.model.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.stereotype.Service;
import tofly.zggas.zggaspay.dao.entity.PrepayEntity;
import tofly.zggas.zggaspay.dao.mapper.PrepayMapper;
import tofly.zggas.zggaspay.properties.PayProperties;
import tofly.zggas.zggaspay.service.entity.dto.PayNotice;
import tofly.zggas.zggaspay.service.entity.dto.TradeBill;
import tofly.zggas.zggaspay.service.entity.dto.WoInfo;

import javax.annotation.PostConstruct;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.security.cert.X509Certificate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
@Slf4j
public class PayServiceImpl implements PayService {

    protected static NativePayService service;
    protected static CertificateService certificateService;
    private final Generator generator;
    private final PrepayMapper prepayMapper;
    private final PayProperties payProperties;

    private static List<X509Certificate> certificates = new ArrayList<>();


    @PostConstruct
    private void init() {
        String mchid = payProperties.getMchid();
        String privateKeyPath = payProperties.getPrivateKeyPath();
        String certSerialNumber = payProperties.getCertSerialNumber();
        String apiV3key = payProperties.getApiV3key();
        try {
            Config config = new RSAAutoCertificateConfig.Builder()
                    .merchantId(mchid)
                    .privateKeyFromPath(privateKeyPath)
                    .merchantSerialNumber(certSerialNumber)
                    .apiV3Key(apiV3key).build();
            // 初始化服务
            service = new NativePayService.Builder().config(config).build();
            certificateService = new CertificateService.Builder().config(config).build();
        } catch (Exception e) {
            log.warn("初始化失败", e);
        }
    }

    @Override
    public PrepayResponse prepay(String woId, Integer total, String attach) {

        if (service == null || certificateService == null) {
            init();
        }

        String outTradeNo = getOutTradeNo();
        String description = payProperties.getDescription();
        String notifyUrl = payProperties.getNotifyUrl();//支付成功通知
        String queryProcedure = payProperties.getQueryWoProcedure();
        if (total == null) {
            if (queryProcedure != null) {
                total = prepayMapper.getPayTotal(queryProcedure, woId);
                if (total < 1) {
                    throw new IllegalArgumentException("工单ID无效");
                }
            } else {
                total = 1;
                log.warn("查询工单金额存储过程未配置");
            }
        }
        String appid = payProperties.getAppid();
        String mchid = payProperties.getMchid();
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime expireTime = now.plusMinutes(1);
        String timeExpire = expireTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss+08:00"));

        PrepayRequest request = new PrepayRequest();
        request.setAppid(appid);
        request.setMchid(mchid);
        Amount amount = new Amount();
        amount.setCurrency("CNY");
        amount.setTotal(total);//单位分

        request.setOutTradeNo(outTradeNo);
        request.setAmount(amount);
        request.setDescription(description);
        request.setNotifyUrl(notifyUrl);
        request.setAttach(attach);
        request.setTimeExpire(timeExpire);

        PrepayResponse response = service.prepay(request);

        PrepayEntity prepayEntity = new PrepayEntity();
        prepayEntity.setOutTradeNo(outTradeNo);
        prepayEntity.setDescription(description);
        prepayEntity.setNotifyUrl(notifyUrl);
        prepayEntity.setTotal(total);
        prepayEntity.setState(0);
        prepayEntity.setWoId(woId);
        prepayEntity.setAttach(attach);
        prepayMapper.insert(prepayEntity);
        return response;
    }

    @Override
    public void check() {
        List<PrepayEntity> unpay = getUnpay();
        if(!unpay.isEmpty()) log.debug("待支付订单:{}", unpay);
        if (service == null) return;
        String mchid = payProperties.getMchid();
        for (PrepayEntity prepayEntity : unpay) {
            try {
                if (prepayEntity.getState() == 1) return;
                QueryOrderByOutTradeNoRequest request = new QueryOrderByOutTradeNoRequest();
                request.setMchid(mchid);
                String outTradeNo = prepayEntity.getOutTradeNo();
                request.setOutTradeNo(outTradeNo);
                Transaction transaction = service.queryOrderByOutTradeNo(request);
                Transaction.TradeStateEnum tradeState = transaction.getTradeState();
                String transactionId = transaction.getTransactionId();
                if (tradeState == Transaction.TradeStateEnum.SUCCESS) {
                    payOkCall(prepayEntity);
                    prepayEntity.setState(1);
                }
                prepayEntity.setTransactionId(transactionId);
                prepayEntity.setTradeState(transaction.getTradeState().name());
                prepayMapper.updateById(prepayEntity);
            } catch (Exception e) {
                log.warn("查询订单异常:{}", e.getMessage());
            }
        }
    }

    @Override
    public String notify_(String signType, String nonce, String timestamp, String requestBody, String signature, String serial) {
        String apiV3key = payProperties.getApiV3key();
        // 构造 RequestParam
        signType = signType == null ? "WECHATPAY2-SHA256-RSA2048" : signType;
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(serial)
                .nonce(nonce)
                .signature(signature)
                .timestamp(timestamp)
                .signType(signType)
                .body(requestBody)
                .build();

        NotificationConfig rsaNotificationConfig = new RSANotificationConfig.Builder()
                .apiV3Key(apiV3key)
                .certificates(getCertificates()).build();
        // 初始化 NotificationParser
        NotificationParser parser = new NotificationParser(rsaNotificationConfig);
        // 验签并解密报文
        PayNotice payNotice = parser.parse(requestParam, PayNotice.class);
        PrepayEntity prepayEntity = prepayMapper.selectById(payNotice.getOutTradeNo());
        if (prepayEntity.getState() == 1) return null;
        String tradeState = payNotice.getTradeState();
        String transactionId = payNotice.getTransactionId();
        if ("SUCCESS".equalsIgnoreCase(payNotice.getTradeState())) {
            payOkCall(prepayEntity);
            prepayEntity.setState(1);
        }
        prepayEntity.setTransactionId(transactionId);
        prepayEntity.setTradeState(tradeState);
        prepayMapper.updateById(prepayEntity);
        return prepayEntity.getOutTradeNo();
    }

    private X509Certificate[] getCertificates() {
        X509Certificate[] certs = new X509Certificate[certificates.size()];
        certificates.toArray(certs);
        return certs;
    }

    private void payOkCall(PrepayEntity prepayEntity) {
        xz(prepayEntity.getWoId(), prepayEntity.getTotal() / 100d, prepayEntity.getOutTradeNo(), prepayEntity.getAttach());
    }

    @Override
    public void closeOrder(String outTradeNo) {
        String mchid = payProperties.getMchid();
        CloseOrderRequest request = new CloseOrderRequest();
        request.setMchid(mchid);
        request.setOutTradeNo(outTradeNo);
        service.closeOrder(request);
    }

    private List<WoInfo> getPaidList(String billDate) {
        String queryPaidWoProcedure = payProperties.getQueryPaidWoProcedure();
        if (queryPaidWoProcedure != null) {
            return this.prepayMapper.getPaidList(queryPaidWoProcedure, billDate);
        } else {
            throw new RuntimeException("查询已下账工单列表存储过程未配置");
        }
    }

    @Override
    public void checkAccount(String billDate) {
        List<TradeBill> tradeBills;
        try {
            DownloadDetail downloadDetail = service.queryTradeBill(billDate, "SUCCESS");
            log.info("账单下载地址:{}", downloadDetail);
            String downloadUrl = downloadDetail.getDownloadUrl();
            try (InputStream inputStream = service.download(downloadUrl)) {
                String tradeBillPath = payProperties.getTradeBillPath();
                File file = new File(tradeBillPath, billDate + ".csv");
                FileUtils.copyInputStreamToFile(inputStream, file);
                tradeBills = getTradeBill(billDate);
                loadWoId(tradeBills);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            //ignore...
        } catch (ServiceException e) {
            if ("NO_STATEMENT_EXIST".equalsIgnoreCase(e.getErrorCode())) {
                tradeBills = new ArrayList<>();
            } else {
                throw e;
            }
        }

        if (log.isDebugEnabled()) {
            log.debug("交易账单:{}", tradeBills);
        }
        List<WoInfo> woInfos = getPaidList(billDate);
        Map<String, WoInfo> successMap = woInfos.stream().collect(Collectors.toMap(WoInfo::getWoId, e -> e));
        List<PrepayEntity> prepayEntities = getByDate(billDate);
        Map<String, PrepayEntity> prepayEntityMap = prepayEntities.stream().collect(Collectors.toMap(PrepayEntity::getOutTradeNo, e -> e));
        for (TradeBill tradeBill : tradeBills) {
            String woId = tradeBill.getWoId();
            PrepayEntity prepayEntity = prepayEntityMap.get(tradeBill.getOutTradeNo());
            if (prepayEntity == null) continue;
            if (!successMap.containsKey(woId)) {
                //业务系统下账
                String dzResult = xz(woId, Double.parseDouble(tradeBill.getTotal()), tradeBill.getOutTradeNo(), prepayEntity.getAttach());
                if ("收费成功".equalsIgnoreCase(dzResult)) {
                    prepayEntity.setDzState(1);
                } else {
                    prepayEntity.setDzState(3);
                }
                prepayEntity.setDzResult(dzResult);
            } else {
                WoInfo woInfo = successMap.get(woId);
                if (woInfo == null) continue;
                if (woInfo.getTotal() == (Double.parseDouble(tradeBill.getTotal()))) {
                    prepayEntity.setDzState(1);
                } else {
                    prepayEntity.setDzState(2);
                    prepayEntity.setDzResult("对账金额不匹配");
                }
            }
        }
        updateDzState(prepayEntities);
        Set<String> successList2 = tradeBills.stream().map(TradeBill::getWoId).collect(Collectors.toSet());
        for (WoInfo woInfo : woInfos) {
            String woId = woInfo.getWoId();
            if (!successList2.contains(woId)) {
                //业务系统重新出账
                cz(woId);
            }
        }
    }

    private void updateDzState(List<PrepayEntity> prepayEntities) {
        if (prepayEntities == null || prepayEntities.isEmpty()) return;
        prepayEntities.forEach(prepayEntity -> this.prepayMapper.update(null, Wrappers.lambdaUpdate(PrepayEntity.class)
                .eq(PrepayEntity::getOutTradeNo, prepayEntity.getOutTradeNo())
                .set(PrepayEntity::getDzState, prepayEntity.getDzState())
                .set(PrepayEntity::getDzResult, prepayEntity.getDzResult())));
    }

    private List<PrepayEntity> getByDate(String billDate) {
        LambdaQueryWrapper<PrepayEntity> queryWrapper = Wrappers.lambdaQuery(PrepayEntity.class).between(PrepayEntity::getCreateTime, billDate + " 00:00:00", billDate + " 23:59:59");
        return prepayMapper.selectList(queryWrapper);
    }

    private void cz(String woId) {
        log.info("出账:{}", woId);
        String czProcedure = payProperties.getCzProcedure();
        if (czProcedure != null) {
            try {
                String result = this.prepayMapper.cz(czProcedure, woId);
                log.info("出账结果:{}", result);
            } catch (Exception e) {
                log.error("出账异常", e);
            }
        } else {
            log.warn("出账存储过程未配置");
        }
    }

    private String xz(String woId, double total, String outTradeNo, String attach) {
        String xzProcedure = payProperties.getXzProcedure();
        log.info("下账:{}", woId);
        if (xzProcedure != null) {
            log.info("{},{},{},{},{}", xzProcedure, woId, total, outTradeNo, attach);
            try {
                String result = this.prepayMapper.xz(xzProcedure, woId, total, outTradeNo, attach);
                log.info("下账结果:{}", result);
                return result;
            } catch (Exception e) {
                log.error("下账异常", e);
                return "下载异常";
            }
        } else {
            log.warn("下账存储过程未配置");
            return "下账存储过程未配置";
        }
    }

    private void loadWoId(List<TradeBill> tradeBills) {
        Set<String> outTradeNos = tradeBills.stream().map(TradeBill::getOutTradeNo).collect(Collectors.toSet());
        if (outTradeNos.isEmpty()) return;
        List<PrepayEntity> prepayEntities = prepayMapper.selectBatchIds(outTradeNos);
        Map<String, String> woIdMap = prepayEntities.stream()
                .collect(Collectors.toMap(PrepayEntity::getOutTradeNo, PrepayEntity::getWoId));
        tradeBills.forEach(e -> e.setWoId(woIdMap.get(e.getOutTradeNo())));
    }

    @Override
    public List<TradeBill> getTradeBill(String billDate) {
        List<TradeBill> tradeBills = new ArrayList<>();
        InputStreamReader in = null;
        try {
            String tradeBillPath = payProperties.getTradeBillPath();
            File file = new File(tradeBillPath, billDate + ".csv");
            if (file.exists() && file.isFile()) {
                in = new InputStreamReader(new FileInputStream(file), StandardCharsets.UTF_8);
                CSVReader csvReader = new CSVReader(in);
                List<String[]> lines = csvReader.readAll();
                int total = lines.size();
                for (int i = 1; i < total - 2; i++) {
                    String[] fields = lines.get(i);
                    tradeBills.add(new TradeBill(trim(fields[6]), trim(fields[18]), trim(fields[0])));
                }
                loadWoId(tradeBills);
            }
        } catch (Exception e) {
            log.error("数据转化失败");
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    // ignore...
                }
            }
        }
        return tradeBills;

    }

    private String trim(String src) {
        return src.substring(1);
    }

    @Override
    public void downloadCertificate() {
        String apiV3key = payProperties.getApiV3key();
        certificates = certificateService.downloadCertificate(apiV3key.getBytes(StandardCharsets.UTF_8));
        //log.trace("平台证书:{}", certificates);
    }

    @Override
    public void test() {
        xz("1C17AF87109E4127994D095D10156C6D", 0.01, "test", "test");
    }

    private List<PrepayEntity> getUnpay() {
        LocalDateTime now = LocalDateTime.now();
        LambdaQueryWrapper<PrepayEntity> queryWrapper = Wrappers.lambdaQuery(PrepayEntity.class).ge(PrepayEntity::getCreateTime, now.plusMinutes(-10)).eq(PrepayEntity::getState, 0);
        return prepayMapper.selectList(queryWrapper);
    }

    private String getOutTradeNo() {
        String outTradeNo = String.valueOf(generator.nextId());
        return String.format("%s%s", LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyMMddHHmmss")), outTradeNo);
    }
}
